#!/usr/bin/env python
import click

from generators import GENERATOR_BY_OUTPUT_FORMAT

@click.command()
@click.option('--title', prompt='Title for the book', type=str)
@click.argument('output_format', default="pdf", type=str)
@click.argument('chapters_dir', default="chapters", type=str)
@click.argument('release_dir', default="release", type=str)
def build_book(title, output_format, chapters_dir, release_dir):
	click.echo(f"Building book with output format {output_format}")
	try:
		book_generator_class = GENERATOR_BY_OUTPUT_FORMAT[output_format]
	except KeyError:
		click.echo(f"Output format {output_format} is currently not supported, consider opening an issue or contributing")
	else:
		click.echo(f"Found parser for {output_format} output type, generating book")
		generator = book_generator_class(title=title, chapters_dir=chapters_dir, release_dir=release_dir)
		generator.build_book()

if __name__ == '__main__':
    build_book()


import os
from abc import ABC, abstractmethod

import pypandoc

from hash import has_hash_changed, write_new_hash


class BookGenerator(ABC):
	def __init__(self, *, title: str, chapters_dir: str, release_dir: str): 
		self.title = title
		self._output_file_name = "_".join(self.title)
		self.chapters_dir = chapters_dir
		self.output_dir = release_dir

	@property
	@abstractmethod
	def FILE_FORMAT(self) -> str:
		"""Output file format of the subclass
		"""
		pass

	def _perform_directory_checks(self):
		if not os.path.isdir(self.chapters_dir):
			raise IOError("Chapters Directory does not exist")
		if not os.path.isdir(self.output_dir):
			os.mkdir(self.output_dir)
		if not os.path.isdir(".hashes"):
			os.mkdir(".hashes")

	def build_book(self):
		print(f"Building book titled {self.title}")
		for (file_data, file_name) in self._read_chapters():
			data = self._format_data_pre_conversion(file_data) or file_data

			print(f"Checking contents for chapter file {file_name}")

			output_file_name = self._get_output_file_name(file_name)
			if has_hash_changed(data, file_name) is True or not os.path.exists(output_file_name):
				print(f"Content has changed for {file_name}, regenerating chapter")
				self._generate_chapter(file_name, data)
				write_new_hash(data, file_name)
			else:
				print(f"Content of {file_name} has not changed, skipping")

	def _read_chapters(self):
		for file_name in os.listdir(self.chapters_dir):
			file_path = f"{self.chapters_dir}/{file_name}"
			with open(file_path, "r") as fr:
				yield (fr.read(), file_name)

	@abstractmethod
	def _format_data_pre_conversion(self, file_data): 
		"""Hook method to tranform data before being pushed to pypandoc
		"""
		pass

	def _get_file_path(self, file_name):
		return f"{self.chapters_dir}/{file_name}"

	def _get_output_file_name(self, file_name):
		base_file_name = file_name.split(".")[0]
		return f"{self.output_dir}/{base_file_name}.pdf"

	def _generate_chapter(self, file_name, file_data):
		file_path = self._get_file_path(file_name)
		output_file_name = self._get_output_file_name(file_name)
		print(f"Generating file {output_file_name} with format {self.FILE_FORMAT}")
		try:
			output = pypandoc.convert_file(file_path, self.FILE_FORMAT, outputfile=output_file_name)
		except RuntimeError as e:
			err = str(e)
			print(f"Encountered error when generating new file, error: {err}")



import hashlib

def has_hash_changed(current_data, filename):
	current_hash = generate_hash(current_data)
	previous_hash = get_previous_hash(filename)
	return not previous_hash == current_hash

def write_new_hash(current_data, filename):
	current_hash = generate_hash(current_data)
	base_file_name = filename.split(".")[0]
	hash_file_path = f".hashes/{base_file_name}.txt"
	with open(hash_file_path, "w+") as fw:
		fw.write(current_hash)

def generate_hash(data):
	return hashlib.sha256(data.encode("utf8")).hexdigest()

def get_previous_hash(filename):
	base_file_name = filename.split(".")[0]
	hash_file_path = f".hashes/{base_file_name}.txt"
	try:
		with open(hash_file_path, "r") as fr:
			return fr.read()
	except FileNotFoundError:
		return None